/***************************************************************************
 *   Copyright (C) 2005 by Andreas Silberstorff   *
 *   ml@andreas-silberstorff.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
 *                                                                         *
 *   In addition, as a special exception, the copyright holders give       *
 *   permission to link the code of this program with any edition of       *
 *   the Qt library by Trolltech AS, Norway (or with modified versions     *
 *   of Qt that use the same license as Qt), and distribute linked         *
 *   combinations including the two.  You must obey the GNU General        *
 *   Public License in all respects for all of the code used other than    *
 *   Qt.  If you modify this file, you may extend this exception to        *
 *   your version of the file, but you are not obligated to do so.  If     *
 *   you do not wish to do so, delete this exception statement from        *
 *   your version.                                                         *
 ***************************************************************************/
#include "kalva_player.h"
#include "kalvacmd.h"
#include "settings.h"

#include <qstring.h>
#include <qlineedit.h>

#include <klocale.h>
#include <kprocess.h>
#include <kinputdialog.h>
#include <kapplication.h>
#include <kmessagebox.h>
#include <kdebug.h>

#include <kdeversion.h>
#undef KDE_3_3_FEATURES
#if defined(KDE_MAKE_VERSION)
#if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
	#define KDE_3_3_FEATURES
#endif
#endif

KalvaPlayer::KalvaPlayer(QWidget *parent, const char *name)
    : kalva_playergui(parent, name)
{
    m_proc = 0;
    playing = false;
}

void KalvaPlayer::slotPlay()
{
    if ( m_proc )
       delete m_proc;
    m_proc = 0;

    kdDebug() << "in slotPlay " << endl;
    kdDebug() << "driver = " << Settings::driver() << endl;

    ///@TODO this is a HACK, make it better!
    if ( Settings::driver() == QString::fromLatin1( i18n( "DVB (experimental!)" ) ) ) {
       cmd   = playCmdDVB();
    }  else  {
       cmd   = playCmd();
    }

    if ( Settings::show_command() == true  ) {
#ifdef KDE_3_3_FEATURES
       bool ok;
       cmd = KInputDialog::getMultiLineText (
          i18n("Command to run"), 
          i18n("Command"), 
          cmd, 
          &ok,
          this,
          "cmdDlg"
       );
       if ( ok != true )  {
          return;
       }
#else
       int ret = KMessageBox::warningContinueCancel (
          this,
          cmd, 
          i18n("Command to run")
       );
       if ( ret != KMessageBox::Continue ) {
          return;
       }
#endif
    }
    m_proc = new KProcess;
    m_proc->setUseShell(true);
    QApplication::connect(
        m_proc, SIGNAL( processExited(KProcess *)),
        this,   SLOT( slotMplayerExited(KProcess *)));
    (*m_proc) << cmd;
    m_proc->start( KProcess::NotifyOnExit, KProcess::Stdin );
}

void KalvaPlayer::slotMplayerExited(KProcess *)
{
    kdDebug() << "In slotMplayerExited() " << endl;
}

void KalvaPlayer::stationChanged(const QString stat, const QString chan, const QString freq)
{
   kdDebug() << "Set channel to " << chan << endl;
   m_station->setText(stat );
   m_channel->setText(chan );
   m_frq->setText(freq );
   if ( m_proc )  {
      kdDebug() << "mplayer process exists" << endl;
      if ( m_proc->isRunning() )  {
         kdDebug() << "Stop mplayer" << endl;
         if ( m_proc->kill() )  {
            kdDebug() << "Restart mplayer" << endl;
            slotPlay();
         }
      }
   }
}

QString KalvaPlayer::playCmd()
{
    kalvaCmd *myCmd = new kalvaCmd( this );
    myCmd->playCmd(   m_channel->text(),
                      m_frq->text()
                     );
    return *myCmd;


    // BUILD the mplayer call
    QString tv_scale = Settings::tv_scale();
    x_scale          = tv_scale.section( ":", 0, 0 );
    y_scale          = tv_scale.section( ":", 1, 1 );
    y_scale          = y_scale.section( " ", 0, 0 );
    QString aumix    = Settings::aumix();
    QString aumixdev = Settings::aumixdev();
    cmd              = "";

   /************************************************
    ************************************************
    ****  optional use of aumix
    ************************************************
    ************************************************/
    if ( !( aumixdev.isEmpty() ) ) {
       cmd.append(QString("%1 -%2 %3 && ")
                 .arg(aumix)
                 .arg( aumixdev )
                 .arg( Settings::audiodev_volume() ) );
    }

    cmd.append(Settings::mplayer());
    cmd.append(" -bpp 32");
    if ( Settings::fullscreen() )
    {
       cmd.append(" -fs");
    }
    cmd.append(" tv:// -tv ");

   /************************************************
    ************************************************
    ****  hardware settings
    ************************************************
    ************************************************/
    cmd.append(QString("driver=%1:norm=%2:device=%3:input=%4")
                  .arg(Settings::driver())
                  .arg(Settings::norm())
                  .arg(Settings::videodev())
                  .arg(Settings::input()));
    int brightness = Settings::brightness();
    if ( brightness != 0 ) {
       cmd.append(QString(":brightness=%1").arg(brightness));
    }
    int hue = Settings::hue();
    if ( hue  != 0 ) {
       cmd.append(QString(":hue=%1").arg(hue));
    }
    int saturation = Settings::saturation();
    if ( saturation != 0 )  {
       cmd.append( QString(":saturation=%1").arg( saturation ));
    }
    int contrast = Settings::contrast();
    if ( contrast != 0 ) {
       cmd.append(QString(":contrast=%1").arg(contrast));
    }

   /************************************************
    ************************************************
    ****  quality settings for the tvcard
    ************************************************
    ************************************************/
    //cmd.append(":outfmt=yuy2:quality=0");
    cmd.append(":quality=0");
    cmd.append(":width=" + x_scale + ":height=" + y_scale);
    //cmd.append(":fps=25");
    if (Settings::use_frq() == true )  {
       cmd.append(":freq=" + m_frq->text());
    }  else  {
       cmd.append(":channel=" + m_channel->text());
    }

   /************************************************
    * free editable en/decoder options
    ************************************************/
    QString postcodingpars = Settings::postcodingpars();
    if ( ! postcodingpars.isEmpty() ) {
       cmd.append(":");
       cmd.append(postcodingpars);
    }

   /************************************************
    ************************************************
    ****  Filters (postprocessing quality settings)
    ************************************************
    ************************************************/

    QString filter = QString::null;

   /************************************************
    * crop
    ************************************************/
    QString crop = Settings::crop();
    if (! crop.isEmpty()) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append("rectangle=" + crop );
    }
   /************************************************
    * free editable filter options
    ************************************************/
    QString prefilter = Settings::prefilter();
    if ( ! filter.isEmpty() ) {
       if (! prefilter.isEmpty()) {
          filter.append(",");
       }
       filter.append(prefilter);
    }
   /************************************************
    * deinterlace, free editable pp options
    ************************************************/
    QString pp = Settings::pp();
    if ( ! pp.isEmpty() ) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append("pp=");
       filter.append(pp);
    }
   /************************************************
    * aspect ratio
    ************************************************/
    if (! filter.isEmpty()) {
       filter.append(",");
    }
    filter.append("dsize=4/3");
   /************************************************
    * scale
    ************************************************/
    QString filterScale = Settings::filter_scale();
    if ((! (filterScale.isEmpty())) and 
        (filterScale != QString::null) and
        (filterScale.contains("\[") <= 0 ))  {
        filterScale    = filterScale.section( " ", 0, 0 );
        if (! filter.isEmpty()) {
           filter.append(",");
        }
        filter.append("scale=" + filterScale);
    }
   /************************************************
    * denoiser
    ************************************************/
    QString denoiser = Settings::denoiser();
    if ( (! denoiser.isEmpty() ) and ( denoiser != "[none]" ) ) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append( denoiser + "=" );
       filter.append( QString("%1:%2:%3")
                           .arg( Settings::denoise3d_chroma() )
                           .arg( Settings::denoise3d_luma() )
                           .arg( Settings::denoise3d_time() )
                           );
    }
   /************************************************
    * free editable filter options
    ************************************************/
    QString postfilter = Settings::postfilter();
    if ( ! postfilter.isEmpty() ) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append(postfilter);
    }

    if (! filter.isEmpty()) {
       cmd.append(" -vf " + filter );
    }

   /************************************************
    ************************************************
    ****  E n d    o f     F i l t e r s
    ************************************************
    ************************************************/

   /************************************************
    ************************************************
    ****  o t h e r     o p t i o n s
    ************************************************
    ************************************************/

    cmd.append(" -vo xv ");


   /************************************************
    ************************************************
    **** verbosity level
    ************************************************
    ************************************************/
    int verbositylevel = Settings::verbosity_level();
    for (int i = 1; i <= verbositylevel; i++ )
    {
       cmd.append(" -v ");
    }

   /************************************************
    ************************************************
    ****  optional use of aumix
    ************************************************
    ************************************************/
    if ( !( aumixdev.isEmpty() ) ) {
       cmd.append(QString(" && %1 -%2 0 ")
                 .arg(aumix)
                 .arg( aumixdev ) );
    }

    return cmd;
}

///@EXPERIMENTAL DVD Support
QString KalvaPlayer::playCmdDVB()
{
    kalvaCmd *myCmd = new kalvaCmd( this );
    myCmd->playCmdDVB( m_station->text()
                     );
    return *myCmd;



    // BUILD the mplayer call

    ///@QUESTION Scaling for DVB possible? Usefull?
    QString tv_scale = Settings::tv_scale();
    x_scale          = tv_scale.section( ":", 0, 0 );
    y_scale          = tv_scale.section( ":", 1, 1 );
    y_scale          = y_scale.section( " ", 0, 0 );

    cmd              = "mplayer ";

    if ( Settings::fullscreen() )
    {
       cmd.append(" -fs");
    }
    QString station = m_station->text();
    cmd.append(QString(" dvb://%1 ").arg( station ) );

    ///@QUESTION these options for DVB possible? Usefull?
     int brightness = Settings::brightness();
     if ( brightness != 0 ) {
        cmd.append(QString(":brightness=%1").arg(brightness));
     }
     int hue = Settings::hue();
     if ( hue  != 0 ) {
        cmd.append(QString(":hue=%1").arg(hue));
     }
     int saturation = Settings::saturation();
     if ( saturation != 0 )  {
        cmd.append( QString(":saturation=%1").arg( saturation ));
     }
     int contrast = Settings::contrast();
     if ( contrast != 0 ) {
        cmd.append(QString(":contrast=%1").arg(contrast));
     }

///QUESTION ???
   /************************************************
    ************************************************
    ****  quality settings for the ???
    ************************************************
    ************************************************/
//    cmd.append(":outfmt=yuy2:quality=0");
//    cmd.append(":quality=0");
//    cmd.append(":width=" + x_scale + ":height=" + y_scale);
//    cmd.append(":fps=25");

   /************************************************
    * free editable en/decoder options
    ************************************************/
    QString postcodingpars = Settings::postcodingpars();
    if ( ! postcodingpars.isEmpty() ) {
       cmd.append(":");
       cmd.append(postcodingpars);
    }

   /************************************************
    ************************************************
    ****  Filters (postprocessing quality settings)
    ************************************************
    ************************************************/

    QString filter = QString::null;

   /************************************************
    * crop
    ************************************************/
    QString crop = Settings::crop();
    if (! crop.isEmpty()) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append("rectangle=" + crop );
    }
   /************************************************
    * free editable filter options
    ************************************************/
    QString prefilter = Settings::prefilter();
    if ( ! filter.isEmpty() ) {
       if (! prefilter.isEmpty()) {
          filter.append(",");
       }
       filter.append(prefilter);
    }
   /************************************************
    * deinterlace, free editable pp options
    ************************************************/
    QString pp = Settings::pp();
    if ( ! pp.isEmpty() ) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append("pp=");
       filter.append(pp);
    }
   /************************************************
    * aspect ratio
    ************************************************/
    if (! filter.isEmpty()) {
       filter.append(",");
    }
    filter.append("dsize=4/3");
   /************************************************
    * scale
    ************************************************/
    QString filterScale = Settings::filter_scale();
    if ((! (filterScale.isEmpty())) and 
        (filterScale != QString::null) and
        (filterScale.contains("\[") <= 0 ))  {
        filterScale    = filterScale.section( " ", 0, 0 );
        if (! filter.isEmpty()) {
           filter.append(",");
        }
        filter.append("scale=" + filterScale);
    }
   /************************************************
    * denoiser
    ************************************************/
    QString denoiser = Settings::denoiser();
    if ( (! denoiser.isEmpty() ) and ( denoiser != "[none]" ) ) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append( denoiser + "=" );
       filter.append( QString("%1:%2:%3")
                           .arg( Settings::denoise3d_chroma() )
                           .arg( Settings::denoise3d_luma() )
                           .arg( Settings::denoise3d_time() )
                           );
    }
   /************************************************
    * free editable filter options
    ************************************************/
    QString postfilter = Settings::postfilter();
    if ( ! postfilter.isEmpty() ) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append(postfilter);
    }

    if (! filter.isEmpty()) {
       cmd.append(" -vf " + filter );
    }

   /************************************************
    ************************************************
    ****  E n d    o f     F i l t e r s
    ************************************************
    ************************************************/

   /************************************************
    ************************************************
    ****  o t h e r     o p t i o n s
    ************************************************
    ************************************************/
    ///@TODO Question: should this be configurable?
    //cmd.append(" -vo xv ");

   /************************************************
    ************************************************
    **** verbosity level
    ************************************************
    ************************************************/
    int verbositylevel = Settings::verbosity_level();
    for (int i = 1; i <= verbositylevel; i++ )
    {
       cmd.append(" -v ");
    }

    return cmd;
}

void KalvaPlayer::play ()
{
   slotPlay();
}

#include "kalva_player.moc"
