//
// C++ Implementation: kalvacmd
//
// Description: 
//
//
// Author: Andreas Silberstorff <andreas@andreas-silberstorff.de>, (C) 2005
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kalvacmd.h"

#include "settings.h"
#include <cmath>
#include <qdir.h>
#include <qstring.h>
#include <kdebug.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include <kmessagebox.h>

#include <kdeversion.h>
#undef KDE_3_3_FEATURES
#if defined(KDE_MAKE_VERSION)
#if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
	#define KDE_3_3_FEATURES
#endif
#endif

kalvaCmd::kalvaCmd( QWidget * p )
        : QString( )
{
     parent = p;
}


kalvaCmd::~kalvaCmd()
{
}

void kalvaCmd::playCmd( const QString & channel,
                        const QString & freq
                      )
{
    m_channel  = channel;
    m_freq     = freq;
    prepareAumix();
    setAumixPrePlay();
    append(Settings::mplayer());
    append(" -bpp 32");
    if ( Settings::fullscreen() )
    {
       append(" -fs");
    }
    setURLtv();
    QString inputOptions = QString::null;
    setInputOptsTV( inputOptions );
    setInputOptsPost( inputOptions );
    if (! inputOptions.isEmpty()) {
       append( " -tv " + inputOptions );
    }

   /************************************************
    ****  F i l t e r s
    ************************************************/
    QString filter = QString::null;
    setFilterRect( filter );
    setFilterPre( filter );
    setFilterPP( filter );
    setFilterScale( filter );
    setFilterAspect( filter );
    setFilterDenoiser( filter );
    setFilterPost( filter );
    if (! filter.isEmpty()) {
       append(" -vf " + filter );
    }
    append(" -vo xv ");
    setVerbosity();
    setAumixPostPlay();
    return;
}

void kalvaCmd::playCmdDVB( const QString & station )
{
    m_station  = station;
    append(Settings::mplayer());
    append(" -bpp 32");
    if ( Settings::fullscreen() )
    {
       append(" -fs");
    }
    setURLdvb();
    QString inputOptions = QString::null;
    setInputOptsPost( inputOptions );
    if (! inputOptions.isEmpty()) {
       append( " -dvbin " + inputOptions );
    }

   /************************************************
    ****  F i l t e r s
    ************************************************/
    QString filter = QString::null;
    setFilterRect( filter );
    setFilterPre( filter );
    setFilterPP( filter );
    setFilterScale( filter );
    setFilterAspect( filter );
    setFilterDenoiser( filter );
    setFilterPost( filter );
    if (! filter.isEmpty()) {
       append(" -vf " + filter );
    }
    //append(" -vo xv ");
    setVerbosity();
    return;
}

void kalvaCmd::recordCmd(           const QString & filmtitle,
                                    const QString & station,
                                    const QString & duration,
                                    const QString & channel,
                                    const QString & freq
                         )
{
    m_channel  = channel;
    m_freq     = freq;
    m_duration = QTime::fromString( duration );

    prepareAumix();
    setAumixPreRec();
    setNice();
    append(Settings::mencoder());
    // URL
    setURLtv();
    QString inputOptions = QString::null;
    setInputOptsTV( inputOptions );
    setInputOptsPost( inputOptions );
    if (! inputOptions.isEmpty()) {
       append( " -tv " + inputOptions );
    }

   /************************************************
    * lavc options
    ************************************************/
    append(" -ovc lavc");
    QString lavcOpts = QString::null;
    setLavcOpts( lavcOpts );
    if ( isEmpty() )
       return;
    setEncoderOptsPost( lavcOpts );
    if (! lavcOpts.isEmpty()) {
       append(" -lavcopts " + lavcOpts );
    }

   /************************************************
    * mp3 audio options
    ************************************************/
    append(" -oac mp3lame -lameopts ");
    append("abr:br=128:mode=0");

   /************************************************
    ****  F i l t e r s
    ************************************************/
    QString filter = QString::null;
    setFilterCrop( filter );
    setFilterPre( filter );
    setFilterPP( filter );
    setFilterScale( filter );
    setFilterDenoiser( filter );
    setFilterPost( filter );
    if (! filter.isEmpty()) {
       append(" -vf " + filter );
    }

   /************************************************
    ************************************************
    **** other options
    ************************************************
    ************************************************/

   /* Mencoder >= 1.0pre7 uses it's own fourcc (FMP4) which makes
      it incompatible with many dvd/divx players. Restore the old
      behaviour by setting it to "DIVX". */
    append(" -ffourcc DIVX");
    append(" -endpos " + duration );

    setTitle( filmtitle, station );
    setVerbosity();

   /************************************************
    **** optional use of aumix
    ************************************************/
    setAumixPostRec();

    return;
}

void kalvaCmd::recordCmdDVBAsDivX4( const QString & filmtitle,
                                    const QString & station,
                                    const QString & duration
                                   )
{
    m_station  = station;
    m_duration = QTime::fromString( duration );
    setNice();
    append(Settings::mencoder());
    setURLdvb();
    QString inputOptions = QString::null;
    setInputOptsPost( inputOptions );
    if (! inputOptions.isEmpty()) {
       append( " -dvbin " + inputOptions );
    }
    append(" -ovc lavc");
    QString lavcOpts = QString::null;
    setLavcOpts( lavcOpts );
    if ( isEmpty() )
       return;
    if (! lavcOpts.isEmpty()) {
       append(" -lavcopts " + lavcOpts );
    }

    /************************************************
     * free editable en/decoder options
     ************************************************/
     QString postlavcopts = Settings::postlavcopts();
     if ( ! postlavcopts.isEmpty() ) {
        append(":");
        append(postlavcopts);
     }

   /************************************************
    * mp3 audio options
    ************************************************/
     append(" -oac mp3lame -lameopts ");
     append("abr:br=128:mode=0");

    /************************************************
    ****  F i l t e r s
    ************************************************/

    QString filter = QString::null;
    setFilterCrop( filter );
    setFilterPre( filter );
    setFilterPP( filter );
    setFilterScale( filter );
    setFilterDenoiser( filter );
    setFilterPost( filter );
    if (! filter.isEmpty()) {
       append(" -vf " + filter );
    }

   /************************************************
    *
    ************************************************/

   /* Mencoder >= 1.0pre7 uses it's own fourcc (FMP4) which makes
      it incompatible with many dvd/divx players. Restore the old
      behaviour by setting it to "DIVX". */
    append(" -ffourcc DIVX");

    append( " -endpos " + duration );
    setTitle( filmtitle, station );
    setVerbosity();
    return;
}

void kalvaCmd::recordCmdDVB(        const QString & filmtitle,
                                    const QString & station,
                                    const QString & duration
                            )
{
    m_station  = station;
    m_duration = QTime::fromString( duration );

    setNice();
    append(Settings::mencoder());
    //URL
    setURLdvb();
    QString inputOptions = QString::null;
    setInputOptsPost( inputOptions );
    if (! inputOptions.isEmpty()) {
       append( " -dvbin " + inputOptions );
    }
    append(" -ovc copy");
    append(" -oac copy ");
    QString filter = QString::null;
    setFilterCrop( filter );
    setFilterPre( filter );
    setFilterScale( filter );
    setFilterPost( filter );
    if (! filter.isEmpty()) {
       append(" -vf " + filter );
    }
    append( " -endpos " + duration );
    append( " -of mpeg" );
    setTitle( filmtitle, station );
    setVerbosity();
    return;
}

/************************************************
 ****  Input options for analogue TV cards
 ************************************************/

void kalvaCmd::setInputOptsTV( QString & Opts )
{
    Opts.append( QString("driver=%1:norm=%2:device=%3:input=%4")
                  .arg(Settings::driver())
                  .arg(Settings::norm())
                  .arg(Settings::videodev())
                  .arg(Settings::input()) );
    // these are rather hardwareoptions as they are used
    // for cards with ugly defaults
    int brightness = Settings::brightness();
    if ( brightness != 0 ) {
       Opts.append(QString(":brightness=%1").arg(brightness));
    }
    int hue = Settings::hue();
    if ( hue  != 0 ) {
       Opts.append(QString(":hue=%1").arg(hue));
    }
    int saturation = Settings::saturation();
    if ( saturation != 0 )  {
       Opts.append(QString(":saturation=%1").arg(saturation));
    }
    int contrast = Settings::contrast();
    if ( contrast != 0 ) {
       Opts.append(QString(":contrast=%1").arg(contrast));
    }
    Opts.append(":quality=0");

    QString tv_scale = Settings::tv_scale();
    x_scale          = tv_scale.section( ":", 0, 0 );
    y_scale          = tv_scale.section( ":", 1, 1 );
    y_scale          = y_scale.section( " ", 0, 0 );
    Opts.append(":width="  + x_scale +
                       ":height=" + y_scale);

    QString tuner = setTuner();
    if ( tuner.isEmpty() )  {
        truncate (0);
        return;
    }
    Opts.append( tuner );
    return;
}

void kalvaCmd::setInputOptsPost( QString & Opts)
{
   /************************************************
    * free editable en/decoder options
    ************************************************/
    QString postcodingpars = Settings::postcodingpars();
    if ( ! postcodingpars.isEmpty() ) {
       if ( ! Opts.isEmpty() )
          Opts.append(":");
       Opts.append(postcodingpars);
    }
    return;
}


/************************************************
 ****  A u m i x
 ************************************************/

void kalvaCmd::prepareAumix()
{
   /************************************************
    **** optional use of aumix
    ************************************************/
    aumix     = Settings::aumix();
    aumixdev  = Settings::aumixdev();
    return;
}

void kalvaCmd::setAumixPrePlay()
{
   /************************************************
    **** optional use of aumix
    ************************************************/
    if ( !( aumixdev.isEmpty() ) ) {
       append(QString("%1 -%2 %3 && ")
                 .arg(aumix)
                 .arg( aumixdev )
                 .arg( Settings::audiodev_volume() ) );
    }
    return;
}

void kalvaCmd::setAumixPostPlay()
{
   /************************************************
    **** optional use of aumix
    ************************************************/
    if ( !( aumixdev.isEmpty() ) ) {
       append(QString(" && %1 -%2 0 ")
                 .arg(aumix)
                 .arg( aumixdev ) );
    }
    return;
}

void kalvaCmd::setAumixPreRec()
{
   /************************************************
    **** optional use of aumix
    ************************************************/
    if ( !( aumixdev.isEmpty() ) ) {
       append(QString("%1 -%2 0 -%3 R -i %4 && ")
                 .arg( aumix )
                 .arg( aumixdev )
                 .arg( aumixdev )
                 .arg( Settings::igain_volume() ) );
    }
    return;
}

void kalvaCmd::setAumixPostRec()
{
   /************************************************
    **** optional use of aumix
    ************************************************/
    if ( !( aumixdev.isEmpty() ) ) {
       append(QString(" && %1 -%2 %3 -%4 P -i 0 ")
                 .arg( aumix )
                 .arg( aumixdev )
                 .arg( Settings::audiodev_volume() )
                 .arg( aumixdev ) );
    }
    return;
}

/************************************************
 ****  
 ************************************************/

void kalvaCmd::setURLtv()
{
    append( QString(" tv:// ") );
}

void kalvaCmd::setURLdvb()
{
    append( QString( " \"dvb://%1\" " ).arg( m_station ) );
}

/************************************************
 ****  Lavc Options
 ************************************************/

void kalvaCmd::setLavcOpts( QString & Opts )
{
   /************************************************
    * 
    ************************************************/
    Opts.append("vcodec=mpeg4");

    setBitrate( Opts );
    if ( isEmpty() )
       return;

    int vratetol = Settings::vratetol();
    if ( vratetol > 0 ) {
       Opts.append(QString(":vratetol=%1").arg(vratetol));
    }

    int mbd = Settings::mbd();
    if ( mbd > 0 ) {
       Opts.append(QString(":mbd=%1").arg(mbd));
    }

    int precmp = Settings::precmp();
    if ( precmp > 0 ) {
       Opts.append(QString(":precmp=%1").arg(precmp));
    }

    int cmp = Settings::cmp();
    if ( cmp > 0 ) {
       Opts.append(QString(":cmp=%1").arg(cmp));
    }

    int subcmp = Settings::subcmp();
    if ( subcmp > 0 ) {
       Opts.append(QString(":subcmp=%1").arg(subcmp));
    }

    QString aspect = Settings::aspect();
    if (! aspect.isEmpty()) {
       Opts.append(QString(":aspect=%1").arg(aspect));
    }

    if ( Settings::use_keyint() == true )  {
       Opts.append(":keyints=" + Settings::keyint() );
    }

    if (Settings::grey() == true)  {
       Opts.append(":gray");
    }
    return;
}

void kalvaCmd::setBitrate( QString & lavcOpts )
{
    double bitrate  = Bitrate();
    if ( bitrate < 0 )  {
       truncate (0);
       return;
    }
    lavcOpts.append(QString(":vbitrate=%1").arg(bitrate));
    return;
}

double kalvaCmd::Bitrate()
{
   double bitrate        = Settings::bitrate();
   double maxBitrate     = Settings::maxBitrate();

   // compute duration in seconds
   int hours   = m_duration.hour();
   int minutes = m_duration.minute();
   int seconds = m_duration.second();
   minutes += (hours * 60);
   if ((minutes > 0) and (seconds > 0))
      minutes -= 1;
   seconds += ( minutes * 60 );

   // test duration
   if ( seconds <= 0 )  {
      KMessageBox::error (
           parent,
           i18n ("Duration is not set") ,
           i18n ("Command not build") );
      return -1;
   }

   // if the hardware profile defines a maximum bitrate
   // the bitrate from the quality profile must not be
   // greater
   if ( maxBitrate && ( maxBitrate < bitrate ) )
       bitrate = maxBitrate;

   // Only test resulting filesize when the user wants this
   if ( ! Settings::testBitrate() )
       return bitrate;

   // compute the maximum bitrate that is allowed if the
   // filesize should stay under 2GB
   double reducedBitrate = std::floor((( 1995 - 0.9375 * minutes) *
                                 8192/( seconds )));

   // test if bitrate is exceeding the computed bitrate
   if ( reducedBitrate < bitrate )  {
      int ret = KMessageBox::Continue;
      if ( Settings::bitrateWarnOnly() )  {
         ret = KMessageBox::warningContinueCancel (
                  parent,
                  i18n("<qt>Using the bitrate <b>%1</b> would result in a <i>filesize &gt;= 2GB</i>. Shall I <i>reduce</i> the bitrate to <b>%2</b> to keep the filesize &lt; 2GB ?</qt>")
                      .arg(bitrate)
                      .arg(reducedBitrate), 
                  i18n("Reduce bitrate?") );
      }
      if ( ret == KMessageBox::Continue )
         bitrate = reducedBitrate;
   }
   return bitrate;
}

void kalvaCmd::setEncoderOptsPost( QString & Opts)
{
   /************************************************
    * free editable en/decoder options
    ************************************************/
    QString postlavcopts = Settings::postlavcopts();
    if ( ! postlavcopts.isEmpty() ) {
       if ( ! Opts.isEmpty() )
          Opts.append( ":" );
       Opts.append( postlavcopts );
    }
    return;
}

/************************************************
 ****  F i l t e r s
 ************************************************/

void kalvaCmd::setFilterRect( QString & filter)
{
   /************************************************
    * crop
    ************************************************/
    QString crop = Settings::crop();
    if (! crop.isEmpty()) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append("rectangle=" + crop );
    }
    return;
}

void kalvaCmd::setFilterCrop( QString & filter)
{
   /************************************************
    * crop
    ************************************************/
    QString crop = Settings::crop();
    if (! crop.isEmpty()) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append("crop=" + crop );
    }
    return;
}

void kalvaCmd::setFilterPre( QString & filter)
{
   /************************************************
    * free editable filter options
    ************************************************/
    QString prefilter = Settings::prefilter();
    if ( ! prefilter.isEmpty() ) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append(prefilter);
    }
    return;
}

void kalvaCmd::setFilterPP( QString & filter)
{
   /************************************************
    * deinterlace, free editable pp options
    ************************************************/
    QString pp = Settings::pp();
    if ( ! pp.isEmpty() ) {
        if (! filter.isEmpty()) {
           filter.append(",");
        }
        filter.append("pp=");
        filter.append(pp);
    }
    return;
}

void kalvaCmd::setFilterScale( QString & filter)
{
   /************************************************
    * scale
    ************************************************/
    QString filterScale = Settings::filter_scale();
    if ( (! (filterScale.isEmpty() ) )  and 
         (filterScale != QString::null ) and
         (filterScale.contains( "\[" ) <= 0 ) )  {
        filterScale    = filterScale.section( " ", 0, 0 );
        if (! filter.isEmpty()) {
            filter.append(",");
        }
        filter.append("scale=" + filterScale);
    }
    return;
}

void kalvaCmd::setFilterAspect( QString & filter )
{
   /************************************************
    * aspect ratio
    ************************************************/
    ///???QString aspect = Settings::aspect();
    ///if ( ! aspect.isEmpty() ) {
       if (! filter.isEmpty()) {
          filter.append(",");
       }
       filter.append("dsize=4/3");
    ///}
    return;
}

void kalvaCmd::setFilterDenoiser( QString & filter)
{
   /************************************************
    * denoiser
    ************************************************/
     QString denoiser = Settings::denoiser();
     if ( (! denoiser.isEmpty() )      and
          ( denoiser != QString::null ) and
          ( denoiser.contains( "\[" ) <= 0 ) )  {
        if (! filter.isEmpty() )
            filter.append(",");
        filter.append( denoiser + "=" );
        filter.append( QString("%1:%2:%3")
                             .arg( Settings::denoise3d_chroma() )
                             .arg( Settings::denoise3d_luma() )
                             .arg( Settings::denoise3d_time() )
                             );
    }
    return;
}

void kalvaCmd::setFilterPost( QString & filter)
{
   /************************************************
    * free editable filter options
    ************************************************/
    QString postfilter = Settings::postfilter();
    if ( ! postfilter.isEmpty() ) {
       if (! filter.isEmpty() ) {
          filter.append( "," );
       }
       filter.append( postfilter );
    }
    return;
}

void kalvaCmd::setNice()
{
   /************************************************
    ************************************************
    **** optional use of nice
    ************************************************
    ************************************************/
    int nice_value    = Settings::nice_value();
    if (nice_value > 0)  {
       append(QString("%1 %2 -%3 ")
                 .arg( Settings::sudo() )
                 .arg( Settings::nice() )
                 .arg( nice_value ) );
    }
    return;
}

void kalvaCmd::setVerbosity()
{
   /************************************************
    ************************************************
    **** verbosity level
    ************************************************
    ************************************************/
    int verbositylevel = Settings::verbosity_level();
    for (int i = 1; i <= verbositylevel; i++ )
    {
       append(" -v ");
    }
    return;
}

void kalvaCmd::setTitle(       const QString & filmtitle,
                                  const QString & station
                          )
{
    QString path      = Settings::storage_path();

    QString myFilmtitle = filmtitle;

    if ( myFilmtitle.isEmpty() )
       myFilmtitle = i18n( "untitled" );

    path.append( QString("/%1").arg( myFilmtitle ) );

    QDir d( path );

    if ( !d.exists() )
    {
       d.mkdir( path );
    }

    QString film = QString("\"%1/%2_`%3 +%a_%d.%b.%G_%H%M%S`")
                      .arg( path )
                      .arg( station )
                      .arg( Settings::date()
                           );

    if ( ( Settings::driver() == QString::fromLatin1( i18n( "DVB (experimental!)" ) ) ) and
         ( ! Settings::convert_mpg2_to_divx() )  ) {
       film.append( ".mpg\"" );
    }  else  {
       film.append( ".avi\"" );
    }
    append( " -o " + film );
    return;
}

QString kalvaCmd::setTuner()
{
    QString tuner = QString::null;
    if ( Settings::use_frq() )  {
       if ( m_freq.isEmpty() )  {
          KMessageBox::error (
               parent,
               i18n ("Frequency is not set") ,
               i18n ("Command not build") );
          truncate (0);
          return QString::null;
       }
       tuner.append(":freq=" + m_freq );
    }  else  {
       if ( m_channel.isEmpty() )  {
          KMessageBox::error (
               parent,
               i18n ("Channel is not set") ,
               i18n ("Command not build") );
          truncate (0);
          return QString::null;
       }
       tuner.append(":channel=" + m_channel );
    }
    return tuner;
}
